/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <errno.h>
#include <glib.h>
#include <dlog.h>
#include <wayland-client.h>
#include <Ecore_Wayland.h>
#include <Elementary.h>
#include <wayland-extension/tizen-extension-client-protocol.h>
#include <tizen-remote-surface-client-protocol.h>

#include "screen_connector_watcher.h"

#ifdef LOG_TAG
#undef LOG_TAG
#endif

#define LOG_TAG "SC_WATCHER"

static struct tizen_remote_surface_manager *__rsm = NULL;
static struct wayland_tbm_client *__tbm_client = NULL;
static struct wl_tbm *__tbm = NULL;
static bool __is_init = false;

struct wl_tbm *screen_connector_get_tbm()
{
	return __tbm;
}

struct tizen_remote_surface_manager *screen_connector_get_rsm()
{
	return __rsm;
}

int screen_connector_trs_init(void)
{
	int ret = 0;
	struct wl_registry *registry;
	Ecore_Wl_Global *global;
	Eina_Inlist *globals;

	if (__is_init)
		return 0;

	ecore_wl_init(NULL);

	registry = ecore_wl_registry_get();
	globals = ecore_wl_globals_get();

	if (!registry || !globals) {
		LOGE("could not get registry(%p) or global list(%p)", registry,
				globals);
		ecore_wl_shutdown();
		return -1;
	}

	EINA_INLIST_FOREACH(globals, global) {
		if (!strcmp(global->interface, "tizen_remote_surface_manager")) {
			__rsm = wl_registry_bind(registry, global->id,
					&tizen_remote_surface_manager_interface,
					global->version < 5 ? global->version : 5);
		}
	}

	if (!__rsm) {
		LOGE("could not get remote surface manager");
		ecore_wl_shutdown();
		return -1;
	}

	__tbm_client = (struct wayland_tbm_client *)wayland_tbm_client_init(ecore_wl_display_get());
	if (!__tbm_client) {
		LOGE("could not get tbm client");
		tizen_remote_surface_manager_destroy(__rsm);
		ecore_wl_shutdown();
		return -1;
	}

	__tbm = (struct wl_tbm *)wayland_tbm_client_get_wl_tbm(__tbm_client);
	if (!__tbm) {
		LOGE("could not get tbm");
		wayland_tbm_client_deinit(__tbm_client);
		tizen_remote_surface_manager_destroy(__rsm);
		ecore_wl_shutdown();
		return -1;
	}

	__is_init = true;

	return ret;
}

int screen_connector_trs_fini(void)
{
	if (!__is_init)
		return 0;

	if (__tbm_client)
		wayland_tbm_client_deinit(__tbm_client);

	if (__rsm)
		tizen_remote_surface_manager_destroy(__rsm);

	ecore_wl_shutdown();

	__is_init = false;
	return 0;
}

